@@ -1,20 +1,4 @@
-"""Graphviz's dot language Python interface.
-
-This module provides with a full interface to
-create handle modify and process graphs in
-Graphviz's dot language.
-
-References:
-
-pydot Homepage: https://github.com/erocarrera/pydot
-Graphviz:       http://www.graphviz.org/
-DOT Language:   http://www.graphviz.org/doc/info/lang.html
-
-Copyright (c) 2005-2011 Ero Carrera <ero.carrera@gmail.com>
-
-Distributed under MIT license
-[http://opensource.org/licenses/mit-license.html].
-"""
+"""An interface to GraphViz."""
 from __future__ import division
 from __future__ import print_function
 import copy
@@ -35,7 +19,7 @@ except Exception as e:
 
 
 __author__ = 'Ero Carrera'
-__version__ = '1.2.3'
+__version__ = '1.2.4'
 __license__ = 'MIT'
 
 
@@ -226,25 +210,26 @@ def quote_if_necessary(s):
 
 
 def graph_from_dot_data(s):
-    """Load graphs from DOT description in (unicode) string `s`.
+    """Load graphs from DOT description in string `s`.
 
-    The data is assumed to be in DOT format. It will
-    be parsed and a Dot class will be returned,
-    representing the graph.
+    @param s: string in [DOT language](
+        https://en.wikipedia.org/wiki/DOT_(graph_description_language))
+
+    @return: Graphs that result from parsing.
+    @rtype: `list` of `pydot.Dot`
     """
     return dot_parser.parse_dot_data(s)
 
 
 def graph_from_dot_file(path, encoding=None):
-    """Load graphs as defined by a DOT file.
+    """Load graphs from DOT file at `path`.
 
-    The file is assumed to be in DOT format. It will
-    be loaded, parsed and a Dot class will be returned,
-    representing the graph.
-
-    @param path: to dot file
-    @param encoding: passed to `io.open`.
+    @param path: to DOT file
+    @param encoding: as passed to `io.open`.
         For example, `'utf-8'`.
+
+    @return: Graphs that result from parsing.
+    @rtype: `list` of `pydot.Dot`
     """
     with io.open(path, 'rt', encoding=encoding) as f:
         s = f.read()
@@ -571,6 +556,8 @@ class Node(Common):
 
         self.create_attribute_methods(NODE_ATTRIBUTES)
 
+    def __str__(self):
+        return self.to_string()
 
 
     def set_name(self, node_name):
@@ -604,8 +591,7 @@ class Node(Common):
 
 
     def to_string(self):
-        """Returns a string representation of the node in dot language.
-        """
+        """Return string representation of node in DOT language."""
 
 
         # RMF: special case defaults for node, edge and graph properties.
@@ -647,8 +633,11 @@ class Edge(Common):
 
     edge(src, dst, attribute=value, ...)
 
-    src: source node's name
-    dst: destination node's name
+    src: source node
+    dst: destination node
+
+    `src` and `dst` can be specified as a `Node` object,
+    or as the node's name string.
 
     All the attributes defined in the Graphviz dot language should
     be supported.
@@ -666,42 +655,29 @@ class Edge(Common):
 
     """
 
-
-
     def __init__(self, src='', dst='', obj_dict=None, **attrs):
-
-        if isinstance(src, (list, tuple)) and dst == '':
-            src, dst = src
-
-        if obj_dict is not None:
-
-            self.obj_dict = obj_dict
-
-        else:
-
-            self.obj_dict = dict()
-
+        self.obj_dict = dict()
+        if isinstance(src, Node):
+            src = src.get_name()
+        if isinstance(dst, Node):
+            dst = dst.get_name()
+        points = (quote_if_necessary(src),
+                  quote_if_necessary(dst))
+        self.obj_dict['points'] = points
+        if obj_dict is None:
             # Copy the attributes
-            #
             self.obj_dict[ 'attributes' ] = dict( attrs )
             self.obj_dict[ 'type' ] = 'edge'
             self.obj_dict[ 'parent_graph' ] = None
             self.obj_dict[ 'parent_edge_list' ] = None
             self.obj_dict[ 'sequence' ] = None
-
-            if isinstance(src, Node):
-                src = src.get_name()
-
-            if isinstance(dst, Node):
-                dst = dst.get_name()
-
-            points = ( quote_if_necessary( src) ,
-                      quote_if_necessary( dst) )
-
-            self.obj_dict['points'] = points
-
+        else:
+            self.obj_dict = obj_dict
         self.create_attribute_methods(EDGE_ATTRIBUTES)
 
+    def __str__(self):
+        return self.to_string()
+
 
     def get_source(self):
         """Get the edges source node name."""
@@ -788,8 +764,7 @@ class Edge(Common):
 
 
     def to_string(self):
-        """Returns a string representation of the edge in dot language.
-        """
+        """Return string representation of edge in DOT language."""
 
         src = self.parse_node_ref( self.get_source() )
         dst = self.parse_node_ref( self.get_destination() )
@@ -915,6 +890,9 @@ class Graph(Common):
 
         self.create_attribute_methods(GRAPH_ATTRIBUTES)
 
+    def __str__(self):
+        return self.to_string()
+
 
     def get_graph_type(self):
 
@@ -1430,9 +1408,10 @@ class Graph(Common):
 
 
     def to_string(self):
-        """Returns a string representation of the graph in dot language.
+        """Return string representation of graph in DOT language.
 
-        It will return the graph and all its subelements in string from.
+        @return: graph and subelements
+        @rtype: `str`
         """
 
 
@@ -1495,7 +1474,7 @@ class Graph(Common):
         obj_list = [(obj['sequence'], obj)
                     for obj in (edge_obj_dicts +
                                 node_obj_dicts + sgraph_obj_dicts) ]
-        obj_list.sort()
+        obj_list.sort(key=lambda x: x[0])
 
         for idx, obj in obj_list:
 
@@ -1635,7 +1614,7 @@ class Cluster(Graph):
         if obj_dict is None:
 
             self.obj_dict['type'] = 'subgraph'
-            self.obj_dict['name'] = 'cluster_'+graph_name
+            self.obj_dict['name'] = quote_if_necessary('cluster_'+graph_name)
 
         self.create_attribute_methods(CLUSTER_ATTRIBUTES)
 
@@ -1675,27 +1654,25 @@ class Dot(Graph):
         # the methods enabling the creation
         # of output in any of the supported formats.
         for frmt in self.formats:
-            self.__setattr__(
-                'create_'+frmt,
-                lambda f=frmt, prog=self.prog:
-                    self.create(format=f, prog=prog))
-            f = self.__dict__['create_'+frmt]
-            f.__doc__ = (
-                "Refer to the docstring accompanying "
-                "the 'create' method for more information.")
+            def new_method(
+                    f=frmt, prog=self.prog,
+                    encoding=None):
+                """Refer to docstring of method `create`."""
+                return self.create(
+                    format=f, prog=prog, encoding=encoding)
+            name = 'create_{fmt}'.format(fmt=frmt)
+            self.__setattr__(name, new_method)
 
         for frmt in self.formats+['raw']:
-            self.__setattr__(
-                'write_'+frmt,
-                lambda path, f=frmt, prog=self.prog:
-                    self.write(path, format=f, prog=prog))
-
-            f = self.__dict__['write_'+frmt]
-            f.__doc__ = (
-                "Refer to the docstring accompanying the "
-                "'write' method for more information.")
-
-
+            def new_method(
+                    path, f=frmt, prog=self.prog,
+                    encoding=None):
+                """Refer to docstring of method `write.`"""
+                self.write(
+                    path, format=f, prog=prog,
+                    encoding=encoding)
+            name = 'write_{fmt}'.format(fmt=frmt)
+            self.__setattr__(name, new_method)
 
     def __getstate__(self):
 
@@ -1742,13 +1719,13 @@ class Dot(Graph):
         self.prog = prog
 
 
-    def write(self, path, prog=None, format='raw'):
+    def write(self, path, prog=None, format='raw', encoding=None):
         """Writes a graph to a file.
 
         Given a filename 'path' it will open/create and truncate
         such file and write on it a representation of the graph
-        defined by the dot object and in the format specified by
-        'format'.
+        defined by the dot object in the format specified by
+        'format' and using the encoding specified by `encoding` for text.
         The format 'raw' is used to dump the string representation
         of the Dot object, without further processing.
         The output can be processed by any of graphviz tools, defined
@@ -1762,28 +1739,33 @@ class Dot(Graph):
 
         which are automatically defined for all the supported formats.
         [write_ps(), write_gif(), write_dia(), ...]
+
+        The encoding is passed to `open` [1].
+
+        [1] https://docs.python.org/3/library/functions.html#open
         """
         if prog is None:
             prog = self.prog
         if format == 'raw':
             s = self.to_string()
-            mode = 'wt'
             if not PY3:
                 s = unicode(s)
+            with io.open(path, mode='wt', encoding=encoding) as f:
+                f.write(s)
         else:
-            s = self.create(prog, format)
-            mode = 'wb'
-        with io.open(path, mode=mode) as f:
-            f.write(s)
+            s = self.create(prog, format, encoding=encoding)
+            with io.open(path, mode='wb') as f:
+                f.write(s)
         return True
 
 
-    def create(self, prog=None, format='ps'):
+    def create(self, prog=None, format='ps', encoding=None):
         """Creates and returns a binary image for the graph.
 
-        create will write the graph to a temporary dot file and process
-        it with the program given by 'prog' (which defaults to 'twopi'),
-        reading the binary image output and return it as:
+        create will write the graph to a temporary dot file in the
+        encoding specified by `encoding` and process it with the
+        program given by 'prog' (which defaults to 'twopi'), reading
+        the binary image output and return it as:
 
         - `str` of bytes in Python 2
         - `bytes` in Python 3
@@ -1851,7 +1833,7 @@ class Dot(Graph):
         # temp file
         tmp_fd, tmp_name = tempfile.mkstemp()
         os.close(tmp_fd)
-        self.write(tmp_name)
+        self.write(tmp_name, encoding=encoding)
         tmp_dir = os.path.dirname(tmp_name)
         # For each of the image files...
         for img in self.shape_files:
@@ -1867,7 +1849,8 @@ class Dot(Graph):
         # explicitly inherit `$PATH`, on Windows too,
         # with `shell=False`
         env = dict()
-        env['PATH'] = os.environ['PATH']
+        env['PATH'] = os.environ.get('PATH', '')
+        env['LD_LIBRARY_PATH'] = os.environ.get('LD_LIBRARY_PATH', '')
         cmdline = [prog, '-T' + format] + args + [tmp_name]
         try:
             p = subprocess.Popen(
@@ -1878,9 +1861,10 @@ class Dot(Graph):
                 stderr=subprocess.PIPE, stdout=subprocess.PIPE)
         except OSError as e:
             if e.errno == os.errno.ENOENT:
-                raise Exception(
-                    '"{prog}" not found in path.'.format(
-                        prog=prog))
+                args = list(e.args)
+                args[1] = '"{prog}" not found in path.'.format(
+                    prog=prog)
+                raise OSError(*args)
             else:
                 raise
         stdout_data, stderr_data = p.communicate()
